import datetime
from .stwno_constants import CSV_MEAL_TYPE_MAP,\
    CSV_NUTRITION_TYPE_MAP,\
    NutritionType,\
    CSV_DATE_FORMAT,\
    MealType,\
    NutritionType,\
    STWNO_INGREDIENTS,\
    STWNO_ALLERGENS


class NoValidDateStringException(Exception):
    pass


class UnknownMealTypeException(Exception):
    pass


class UnknownNutritionTypeException(Exception):
    pass


class UnknownIngredientException(Exception):
    pass


class StwnoFoodIngredient():
    identifier: str
    name: str

    def __init__(self, identifier, name):
        self.name = name
        self.identifier = identifier


def convertCSVDishName(name: str) -> str:
    name = name.strip()
    indexBracket = name.find('(')
    if indexBracket == -1:
        return name
    return name[0:indexBracket].strip()


def convertCSVMealType(mealType: str) -> MealType:
    mealTypeID = ''.join(filter(str.isalpha, mealType))
    mType = CSV_MEAL_TYPE_MAP.get(mealTypeID)
    if mType != None:
        return mType
    else:
        raise UnknownMealTypeException(
            'MealType {} is not in the list of known meal types'.format(mType))


def convertCSVNutritionType(nutritionType: str) -> NutritionType:
    nutType = CSV_NUTRITION_TYPE_MAP.get(nutritionType)
    if nutType != None:
        return nutType
    else:
        return NutritionType.meat


def convertCSVDate(dateStr: str) -> datetime.date:
    try:
        return datetime.datetime.strptime(dateStr, CSV_DATE_FORMAT).date()
    except ValueError:
        raise NoValidDateStringException


def convertCSVIngredientsAndAllergens(mealNameStr: str) -> str:
    ingredientIdentifiers = mealNameStr.replace(' ', '').split(',')
    ingredients = []
    allergens = []
    for identifier in ingredientIdentifiers:
        name = None
        if name := STWNO_INGREDIENTS.get(identifier):
            ingredients.append(StwnoFoodIngredient(identifier, name))
        elif name := STWNO_ALLERGENS.get(identifier):
            allergens.append(StwnoFoodIngredient(identifier, name))
        else:
            raise UnknownIngredientException(
                'Ingredient {} is neither a known inredient nor an allergene'.format(identifier))
    return ingredients, allergens





from .stwno_constants import Canteen
from .csv_converters import UnknownIngredientException,\
    UnknownMealTypeException,\
    UnknownNutritionTypeException,\
    NoValidDateStringException,\
    StwnoFoodIngredient
from .stwno_canteen import getMenu,\
    StwnoCanteenPricing,\
    Dish

if __name__ == '__main__':
    print('hi')



from enum import Enum

CSV_DATE_FORMAT = '%d.%m.%Y'
CSV_HEADER_LINE_CNT = 1  # header lines get removed before processing

CSV_INDEX_DATE = 0
CSV_INDEX_MEAL_TYPE = 2
CSV_INDEX_MEAL_AND_META = 3
CSV_INDEX_NUTRITION_TYPE = 4
CSV_INDEX_STUDENT_PRICE = 6
CSV_INDEX_PERSONNEL_PRICE = 7
CSV_INDEX_GUEST_PRICE = 8


class Canteen(Enum):
    UNI_RGB_CANTEEN = 1
    UNI_RGB_GUEST_CANTEEN = 2
    UNI_RGB_CAFETERIA_PT = 3
    UNI_RGB_CAFETERIA_CHEMISTRY = 4
    UNI_RGB_CAFETERIA_SPORT = 5
    OTH_RGB_CANTEEN_LUNCH = 6
    OTH_RGB_CANTEEN_DINNER = 7
    OTH_RGB_CAFETERIA_PRUEFENING = 8
    UNI_PAR_CANTEEN = 9
    UNI_PAR_NIKOLA_MONASTERY_CAFETERIA = 10
    HS_DEG_CANTEEN = 11
    HS_LA_CANTEEN = 12
    HS_SR_CANTEEN = 13
    HS_PAN_CANTEEN = 14


CANTEENS = {
    Canteen.UNI_RGB_CANTEEN: 'UNI-R',
    Canteen.UNI_RGB_GUEST_CANTEEN: 'UNI-R-Gs',
    Canteen.UNI_RGB_CAFETERIA_PT: 'Cafeteria-PT',
    Canteen.UNI_RGB_CAFETERIA_CHEMISTRY: 'Cafeteria-Chemie',
    Canteen.UNI_RGB_CAFETERIA_SPORT: 'Cafeteria-Sport',
    Canteen.OTH_RGB_CANTEEN_LUNCH: 'HS-R-tag',
    Canteen.OTH_RGB_CANTEEN_DINNER: 'HS-R-abend',
    Canteen.OTH_RGB_CAFETERIA_PRUEFENING: 'Cafeteria-pruefening',
    Canteen.UNI_PAR_CANTEEN: 'UNI-P',
    Canteen.UNI_PAR_NIKOLA_MONASTERY_CAFETERIA: 'Cafeteria-Nikolakloster',
    Canteen.HS_DEG_CANTEEN: 'HS-DEG',
    Canteen.HS_LA_CANTEEN: 'HS-LA',
    Canteen.HS_SR_CANTEEN: 'HS-RA',
    Canteen.HS_PAN_CANTEEN: 'HS-PAN'
}


class NutritionType(Enum):
    vegan = 1
    vegetarian = 2
    meat = 3


class MealType(Enum):
    MainDish = 1
    SideDish = 2
    Dessert = 3


CSV_MEAL_TYPE_MAP = {
    'HG': MealType.MainDish,
    'B': MealType.SideDish,
    'N': MealType.Dessert
}

CSV_NUTRITION_TYPE_MAP = {
    'V': NutritionType.vegetarian,
    'S': NutritionType.vegetarian,
    'VG': NutritionType.vegan,
    'M': NutritionType.meat,
    'F': NutritionType.meat
}

CSV_ADDR_FORMAT_STR = 'http://www.stwno.de/infomax/daten-extern/csv/{}/{}.csv'

STWNO_INGREDIENTS = {
    '1': 'colorants',
    '2': 'preservatives',
    '3': 'antioxidants',
    '4': 'flavor enhancers',
    '5': 'sulfured',
    '6': 'blackened',
    '7': 'waxed',
    '8': 'phosphate',
    '9': 'contains sweetener saccharine',
    '10': 'containes sweetener aspartam',
    '11': 'contains sweetener cyclamat',
    '12': 'contains sweetener acesulfam',
    '13': 'quinine',
    '14': 'caffeine',
    '15': 'genetically changed',
    '16': 'contains sulfates',
    '17': 'contains phenylalanin',
}

STWNO_ALLERGENS = {
    'AA': 'wheat gluten',
    'AB': 'rye gluten',
    'AC': 'barley gluten',
    'AD': 'oat gluten',
    'AE': 'spelt gluten',
    'AF': 'kamut gluten',
    'B': 'curstaceans',
    'C': 'eggs',
    'D': 'fish',
    'E': 'peanuts',
    'F': 'soy',
    'G': 'milk and milk products',
    'HA': 'almond',
    'HB': 'hazelnut',
    'HC': 'walnut',
    'HD': 'cashew',
    'HE': 'peca nut',
    'HF': 'brazil nut',
    'HG': 'pistachio',
    'HH': 'macadamia',
    'HI': 'queenslandnuss',
    'I': 'celeriac',
    'J': 'senap',
    'K': 'sesame seeds',
    'L': 'sulfur dioxide and sulfites',
    'M': 'lupines',
    'N': 'molluscs',
    'O': 'nitrate',
    'P': 'nitrate curing salt'
}



from distutils.core import setup
setup(
  name = 'stwno_canteen',
  packages = ['stwno_python'],
  version = '0.1',
  license='MIT',
  description = 'simple API to access the STWNO canteen\'s menus',
  author = 'Stefan Sommer aka somsky',
  author_email = 'stefan.sommer.per@gmail.com',
  url = 'https://github.com/somsky/stwno_canteen',
  download_url = 'https://github.com/user/reponame/archive/v_01.tar.gz',
  keywords = ['stwno'],
  install_requires=[],
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.8',
  ],
)



import urllib.request
import datetime
from enum import Enum
import csv
import logging
from typing import List
import re
import argparse
from .stwno_constants import Canteen,\
    CANTEENS,\
    CSV_ADDR_FORMAT_STR,\
    NutritionType,\
    MealType,\
    CSV_MEAL_TYPE_MAP,\
    CSV_NUTRITION_TYPE_MAP,\
    CSV_INDEX_DATE,\
    CSV_INDEX_MEAL_TYPE,\
    CSV_INDEX_MEAL_AND_META,\
    CSV_INDEX_NUTRITION_TYPE,\
    CSV_INDEX_STUDENT_PRICE,\
    CSV_INDEX_PERSONNEL_PRICE,\
    CSV_INDEX_GUEST_PRICE,\
    CSV_HEADER_LINE_CNT

from .csv_converters import convertCSVDate,\
    convertCSVDishName,\
    convertCSVMealType,\
    convertCSVNutritionType,\
    convertCSVIngredientsAndAllergens,\
    StwnoFoodIngredient


class StwnoCanteenPricing():
    studentPrice: float
    personnelPrice: float
    guestPrice: float

    def __init__(self, studentPrice: float, personnelPrice: float, guestPrice: float):
        self.studentPrice = studentPrice
        self.personnelPrice = personnelPrice
        self.guestPrice = guestPrice


class Dish():
    name: str
    ingredients: List[StwnoFoodIngredient]
    allergens: List[StwnoFoodIngredient]
    mealType: MealType
    pricing: StwnoCanteenPricing
    servedOn: datetime.date
    nutritionType: NutritionType

    def __init__(self, name: str, servedOn, nutritionType: NutritionType, ingredients, allergens, mealType, pricing):
        self.name = name
        self.servedOn = servedOn
        self.ingredients = ingredients
        self.allergens = allergens
        self.mealType = mealType
        self.pricing = pricing


def queryStwnoMenu(isoWeekNumber: int, canteen: Canteen) -> str:
    canteenID = CANTEENS[canteen]
    csvAddr = CSV_ADDR_FORMAT_STR.format(canteenID, isoWeekNumber)
    csvData = urllib.request.urlopen(csvAddr)
    csvString = csvData.read().decode('latin-1')
    return csvString


def getMenu(canteen: Canteen, day: datetime.date) -> List[Dish]:
    '''
    Get the menu of a STWNO canteen for a specific day

    Parameters
    -----------
        canteen (Canteen): The canteen to get the menu for
        day (datetime.date): The day of the menu

    Raises
    -------
        NoValidDateStringException
            The menu date returned from the Stwno api contained a malformatted date
        UnknownMealTypeException
            The meal type of a menu item returned from the Stwno api is unknown
        UnknownNutritionTypeException
            The nutrition type of a menu item returned from the Stwno api is unknown

    Returns
    --------
        The dishes served on the specified day in the specified Stwno canteen

    '''

    isoWeekNumber = day.isocalendar()[1]
    csvMenu = queryStwnoMenu(isoWeekNumber, canteen)
    csvReader = csv.reader(csvMenu.splitlines(), delimiter=';')
    rawMenu = list(csvReader)[CSV_HEADER_LINE_CNT:]
    dishes = []
    for rawDish in rawMenu:
        servedOn = convertCSVDate(rawDish[CSV_INDEX_DATE])
        dishName = convertCSVDishName(rawDish[CSV_INDEX_MEAL_AND_META])
        try:
            ingredients, allergens = convertCSVIngredientsAndAllergens(
                rawDish[CSV_INDEX_MEAL_AND_META])
        except:
            ingredients = allergens = []
        mealType = convertCSVMealType(rawDish[CSV_INDEX_MEAL_TYPE])
        nutritionType = convertCSVNutritionType(rawDish[CSV_INDEX_NUTRITION_TYPE])
        studentPrice = float(rawDish[CSV_INDEX_STUDENT_PRICE].replace(',', '.'))
        personnelPrice = float(rawDish[CSV_INDEX_PERSONNEL_PRICE].replace(',', '.'))
        guestPrice = float(rawDish[CSV_INDEX_GUEST_PRICE].replace(',', '.'))
        price = StwnoCanteenPricing(studentPrice, personnelPrice, guestPrice)
        dishes.append(Dish(dishName, servedOn, nutritionType,
                           ingredients, allergens, mealType, price))
    dishes = list(filter(lambda dish: dish.servedOn == day, dishes))
    return dishes




import argparse
import datetime
from .stwno_canteen import getMenu
from .stwno_constants import Canteen

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='STWNO canteen API. Main purpose is to be used\
            as a API in other python scripts, although it supports basic command line functionality.\
            Calling this module directly makes it possible to display the menu for the current day\
            for a given canteen.')
    parser.add_argument('-c', '--canteen', required=True,
                        help='ID of the canteen to fetch a menu from. The correct identifier can be\
                        obtained by browsing the stwno website or by looking at the stwno-constants file')
    args = parser.parse_args()
    menu = getMenu(Canteen.OTH_RGB_CANTEEN_LUNCH, datetime.date.today())
    for item in menu:
        print('{:<60} {:.2f}€'.format(item.name, item.pricing.studentPrice))



import setuptools
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README").read_text()

# This call to setup() does all the work
setuptools.setup(
  name = 'stwno_canteen',
  version = '0.1',
  license='MIT',
  description = 'simple API to access the STWNO canteen\'s menus',
  long_description=README,
  long_description_content_type='text/markdown',
  author = 'Stefan Sommer aka somsky',
  author_email = 'stefan.sommer.per@gmail.com',
  url = 'https://github.com/somsky/stwno_canteen',
  keywords = ['stwno'],
  install_requires=[],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.8',
  ],
  packages=['stwno_canteen'],
  python_requires='>=3.8'
)
